Import "TGameElement.bmx"
Import "TCoordinate.bmx"

Incbin "gfx/whitecircle.png"

Type TBullet Extends TGameElement

	Global bulletImg:TImage
	Global allBullets:TList = New TList
	Global defaultspeed:Float = 3.0	' with a logical framerate of 100 and a speed of 3.0 our bullets move 300 pixel per second
	
	Field dir:TCoordinate
	Field speed:Float
	Field shotBy:Int
	

	Function Create:TBullet(pos:TCoordinate, direction:TCoordinate, speed:Float)
		Local bullet:TBullet = New TBullet
		bullet.pos = TCoordinate.Copy(pos)
		bullet.oldPos = TCoordinate.Copy(pos)
		If bulletImg = Null
			bulletImg = LoadImage("incbin::gfx/whitecircle.png")
		End If
		bullet.angle = direction.angle
		bullet.dir = direction
		bullet.speed = speed
		allBullets.AddLast(bullet)
		Return bullet
	End Function

		
	Method Update(fixedRate:Float)
		' move in direction dir with given bullet speed
		Self.oldPos.SetLike(Self.pos) ' store current pos in oldPos
		Self.pos.x:+(Self.dir.x * Self.speed)
		Self.pos.y:+(Self.dir.y * Self.speed)
		
		If (Self.pos.x < 0 Or Self.pos.y < 0 Or Self.pos.x > GraphicsWidth()  Or Self.pos.y > GraphicsHeight() or self.dead)
			' delete this bullet
			allBullets.Remove(Self)
		End If
	End Method

		
	Method Render(tween:Float)
		SetRotation(Self.angle)
		' calculate proper tween position here
		SetColor(255, 0, 0)
		DrawImage(bulletImg, TGameElement.Tween(oldpos.x, pos.x, tween), TGameElement.Tween(oldpos.y, pos.y, tween))
		SetRotation(0)
		SetColor(255, 255, 255)
	End Method

		
	Function UpdateAllBullets(fixedRate:Float)
		For Local bullet:TBullet = EachIn allBullets
			bullet.Update(fixedRate)
		Next
	End Function

		
	Function RenderAllBullets(tween:Float)
		For Local bullet:TBullet = EachIn allBullets
			bullet.Render(tween)
		Next
		' DrawText "Nr of bullets " + allBullets.Count(), 10, 400
	End Function
End Type
