Type TCoordinate
	Field x:Float
	Field y:Float
	Field angle:Int	' we simplify here and just use an integer for angles between 0 and 359 degrees
	
	Function Create:TCoordinate(x:Float, y:Float, angle:Int)
		Local coord:TCoordinate = New TCoordinate
		coord.x = x
		coord.y = y
		coord.angle = angle
		Return coord
	End Function
	
	
	Function Copy:TCoordinate(original:TCoordinate)
		Local copy:TCoordinate = New TCoordinate
		copy.x = original.x
		copy.y = original.y
		copy.angle = original.angle
		Return copy
	End Function
	
	Method SetPosition(x:Float, y:Float)
		Self.x = x
		Self.y = y
	End Method
	
	Method SetPositionLike(otherPos:TCoordinate)
		Self.x = otherPos.x
		Self.y = otherPos.y
	End Method
	
	Method SetAngle(angle:Int)
		self.angle = angle
	End Method
	
	Method SetAngleLike(otherPos:TCoordinate)
		Self.angle = otherPos.angle
	End Method
	
	Method SetLike(otherPos:TCoordinate)
		SetPositionLike(otherPos)
		SetAngleLike(otherPos)
	End Method
	
	Method GetDistance:Float(otherPos:TCoordinate)
		Local l1:Float = Abs(Self.x - otherPos.x)
		Local l2:Float = Abs(Self.y - otherPos.y)
		Return Sqr((l1 * l1) + (l2 * l2))
	End Method
	
	Method GetAngle:Int(otherPos:TCoordinate)
         Local dx:Float = otherPos.x - Self.x
         Local dy:Float = otherPos.y - Self.y
         Return Int((ATan2(dy:Float, dx:Float) + 360) Mod 360)
	End Method
	
	Rem
	BBDoc: calculate a normalized direction vector pointing from self to given toPos
	EndRem
	Method GetDirection:TCoordinate(toPos:TCoordinate)
		Local xdiff:Float = toPos.x - Self.x
		Local ydiff:Float = toPos.y - Self.y
		Local angle:Int = Self.GetAngle(toPos)
		Local dir:TCoordinate = TCoordinate.Create(xdiff, ydiff, angle)
		dir.Normalize()
		Return dir
	End Method
	
	Rem
	BBDoc: normalizes this vector
	endrem
	Method Normalize()
		Local magnitude:Float = Sqr(Self.x * Self.x + Self.y * Self.y)
		Self.x = Self.x / magnitude
		Self.y = Self.y / magnitude
	End Method
	
End Type
