rem
	bbdoc: Type description
end rem
Import "TGameElement.bmx"
Import "TCoordinate.bmx"
Import "TBullet.bmx"

Incbin "gfx/soldier.png"

Type TEnemy Extends TGameElement
	Global enemyImg:TImage
	Global allEnemies:TList = New TList
	Global defaultspeed:Float = 1.0	' with a logical framerate of 100 and a speed of 1.0 our enemies move 100 pixel per second
	Global frameChange:Int = 20 ' with a logical framerate of 100 we change the animation frame every fifth of a second
	Global frameCount:Int = 0
	Global frameInc:Int = 1
	
	Field dir:TCoordinate
	Field speed:Float
	Field frame:Int = 0	' first frame of animated image that we show


	Function Create:TEnemy(pos:TCoordinate, direction:TCoordinate, speed:Float)
		Local enemy:TEnemy = New TEnemy
		enemy.pos = TCoordinate.Copy(pos)
		enemy.oldPos = TCoordinate.Copy(pos)
		If enemyImg = Null
			enemyImg = LoadAnimImage("incbin::gfx/soldier.png", 40, 40, 0, 3)
		End If
		enemy.angle = direction.angle
		enemy.dir = direction
		enemy.speed = speed
		allEnemies.AddLast(enemy)
		Return enemy
	End Function

	
	Method Update(fixedRate:Float)
		' move in direction dir with given enemy speed
		Self.oldPos.SetLike(Self.pos) ' store current pos in oldPos
		Self.pos.x:+(Self.dir.x * Self.speed)
		Self.pos.y:+(Self.dir.y * Self.speed)
		Self.frameCount:+1
		If Self.frameCount > Self.frameChange
			Self.frameCount = 0
			Self.frame:+Self.frameInc
			If Self.frame > 2
				Self.frame = 0
				Self.frameInc = -Self.frameInc
			End If
			If Self.frame < 0
				Self.frame = 2
				Self.frameInc = -Self.frameInc
			End If
		End If

		' check for collisions with bullets
		For Local bullet:TBullet = EachIn TBullet.allBullets
			If ImagesCollide(Self.enemyImg, Self.pos.x, Self.pos.y, Self.frame, bullet.bulletImg, bullet.pos.x, bullet.pos.y, 0)
				Self.dead = True
				bullet.dead = True
			End If
		Next
		
		If (Self.pos.x < 0 Or Self.pos.y < 0 Or Self.pos.x > GraphicsWidth() Or Self.pos.y > GraphicsHeight() Or Self.dead)
			' delete this enemy
			allEnemies.Remove(Self)
		End If
		
	End Method

		
	Method Render(tween:Float)
		SetRotation(Self.angle)
		' calculate proper tween position here
		SetColor(255, 255, 255)
		DrawImage(enemyImg, TGameElement.Tween(oldpos.x, pos.x, tween), TGameElement.Tween(oldpos.y, pos.y, tween), Self.frame)
		SetRotation(0)
		SetColor(255, 255, 255)
	End Method

		
	Function UpdateAllEnemies(fixedRate:Float, playerPos:TCoordinate)
		If allEnemies.Count() = 0
			Local randomvalue = Rand(0, 100)
			If randomvalue > 95
				Local borderPos:TCoordinate = GetRandomBorderPos()
				Local dirToPlayer:TCoordinate = borderPos.GetDirection(playerPos)
				TEnemy.Create(borderPos, dirToPlayer, defaultspeed)
			End If
		End If

		For Local enemy:TEnemy = EachIn allEnemies
			enemy.Update(fixedRate)
		Next
	End Function
	
	Function GetRandomBorderPos:TCoordinate()
		Local border:Int = Rand(1, 4)
		Local pos:TCoordinate, xp:Int, yy:Int
		
		Select border
			Case 1 'from the top
				yp = 0
				xp = Rand(0, GraphicsWidth() - 40)
			Case 2 'from below
				yp = GraphicsHeight()
				xp = Rand(0, GraphicsWidth() - 40)
			Case 3 ' from the left
				xp = 0
				yp = Rand(0, GraphicsHeight() - 40)
			Case 4 ' from the right
				xp = GraphicsWidth()
				yp = Rand(0, GraphicsHeight() - 40)
		End Select
		pos = TCoordinate.Create(xp, yp, 0)
		Return pos
	End Function

		
	Function RenderAllEnemies(tween:Float)
		For Local enemy:TEnemy = EachIn allEnemies
			enemy.Render(tween)
		Next
		' DrawText "Nr of enemies " + allEnemies.Count(), 10, 430
	End Function
		
End Type
